#!/usr/bin/env python3
"""
Qwen3-Embedding 服务器
使用 HuggingFace sentence-transformers + FastAPI 运行 embedding 服务
支持 Qwen3-Embedding-0.6B 和 Qwen3-Embedding-4B 模型

依赖安装:
    pip install fastapi uvicorn sentence-transformers torch

使用方法:
    # 使用 Qwen3-Embedding-0.6B (默认 CPU)
    python embedding_server.py

    # 使用 GPU
    python embedding_server.py --device cuda

    # 使用 Qwen3-Embedding-4B (需要 >10GB 显存)
    python embedding_server.py --device cuda --model-path /path/to/Qwen3-Embedding-4B

    # 指定 GPU
    python embedding_server.py --device cuda:1

    # 后台运行
    nohup python embedding_server.py > server.log 2>&1 &
"""

import os
import sys
import argparse
import logging
import time
from typing import List, Optional, Union

import torch
from fastapi import FastAPI, HTTPException
from pydantic import BaseModel
import uvicorn

# 尝试导入 sentence-transformers
try:
    from sentence_transformers import SentenceTransformer
except ImportError:
    logger.error("需要安装 sentence-transformers")
    logger.error("请运行：pip install sentence-transformers")
    sys.exit(1)

# 配置日志
def setup_logging(service_name: str):
    """配置日志，输出到控制台和文件"""
    logs_dir = os.path.expanduser("~/.cache/pi-llm-server/logs")
    os.makedirs(logs_dir, exist_ok=True)

    log_file = os.path.join(logs_dir, f"{service_name}.log")

    # 创建 logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    # 清除已有的 handlers
    logger.handlers = []

    # 创建 formatter
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    # 控制台 handler
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

    # 文件 handler
    file_handler = logging.FileHandler(log_file, encoding='utf-8')
    file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    return logger

logger = setup_logging("embedding")

# FastAPI 应用
app = FastAPI(
    title="Qwen3-Embedding API",
    description="Qwen3-Embedding 模型 embedding 服务",
    version="1.0.0"
)

# 全局变量
model = None
device = None
model_name = None

# 默认配置
DEFAULT_MODEL_PATH = os.path.expanduser("~/.cache/modelscope/hub/models/unsloth/Qwen3-Embedding-0.6B")
DEFAULT_HOST = "0.0.0.0"
DEFAULT_PORT = 8091


# ==================== 请求/响应模型 ====================

class EmbeddingRequest(BaseModel):
    input: Union[str, List[str]]
    model: str = ""
    encoding_format: Optional[str] = "float"


class EmbeddingData(BaseModel):
    object: str = "embedding"
    embedding: List[float]
    index: int


class EmbeddingUsage(BaseModel):
    prompt_tokens: int
    total_tokens: int


class EmbeddingResponse(BaseModel):
    object: str = "list"
    data: List[EmbeddingData]
    model: str
    usage: EmbeddingUsage


class SimilarityRequest(BaseModel):
    text1: str
    text2: str


# ==================== 模型加载 ====================

def load_model(model_path: str, device_str: str = "cuda"):
    """加载 embedding 模型"""
    global model, device, model_name

    # 从路径提取模型名
    model_name = os.path.basename(os.path.dirname(model_path)) + "/" + os.path.basename(model_path)

    # 判断设备
    use_cuda = device_str.startswith("cuda") and torch.cuda.is_available()

    if device_str.startswith("cuda") and not torch.cuda.is_available():
        logger.warning("未检测到 GPU，自动切换到 CPU")

    if use_cuda:
        gpu_idx = 0
        if ":" in device_str:
            gpu_idx = int(device_str.split(":")[1])
        gpu_name = torch.cuda.get_device_name(gpu_idx)
        gpu_memory = torch.cuda.get_device_properties(gpu_idx).total_memory / 1024**3
        logger.info(f"检测到 GPU: {gpu_name} ({gpu_memory:.1f} GB)")

    logger.info(f"正在加载模型：{model_path}")

    try:
        if use_cuda:
            logger.info(f"尝试使用 {device_str} 加载...")
            model = SentenceTransformer(
                model_path,
                device=device_str,
                trust_remote_code=True
            )
            device = torch.device(device_str)
            logger.info(f"模型加载成功，使用 GPU: {gpu_name}")
        else:
            logger.info("使用 CPU 加载...")
            model = SentenceTransformer(
                model_path,
                device="cpu",
                trust_remote_code=True,
                model_kwargs={"low_cpu_mem_usage": True}
            )
            device = torch.device("cpu")
            logger.info("模型加载成功，使用 CPU")

    except torch.OutOfMemoryError as e:
        if use_cuda:
            logger.warning(f"GPU 显存不足，切换到 CPU: {e}")
            model = SentenceTransformer(
                model_path,
                device="cpu",
                trust_remote_code=True,
                model_kwargs={"low_cpu_mem_usage": True}
            )
            device = torch.device("cpu")
            logger.info("模型已在 CPU 上加载")
        else:
            raise

    # 显示模型信息
    total_params = sum(p.numel() for p in model.parameters())
    embedding_dim = model.get_sentence_embedding_dimension()
    logger.info(f"模型参数：{total_params / 1e6:.1f}M, 向量维度：{embedding_dim}")


# ==================== API 端点 ====================

@app.get("/health")
async def health_check():
    """健康检查"""
    return {"status": "healthy", "device": str(device) if device else "not_loaded"}


@app.get("/v1/models")
async def list_models():
    """列出可用模型"""
    return {
        "object": "list",
        "data": [
            {
                "id": model_name,
                "object": "model",
                "owned_by": "local"
            }
        ]
    }


@app.post("/v1/embeddings")
async def create_embeddings(request: EmbeddingRequest):
    """OpenAI 兼容的 embedding 接口"""
    if model is None:
        raise HTTPException(status_code=503, detail="模型未加载")

    try:
        # 处理输入
        if isinstance(request.input, str):
            input_texts = [request.input]
        else:
            input_texts = request.input

        if not input_texts:
            raise HTTPException(status_code=400, detail="输入文本不能为空")

        start_time = time.time()

        # 生成 embedding
        embeddings = model.encode(
            input_texts,
            convert_to_tensor=False,
            show_progress_bar=False
        )

        elapsed = time.time() - start_time
        logger.info(f"处理 {len(input_texts)} 条文本，耗时 {elapsed:.3f}s")

        # 精确计算 token 数
        tokenizer = model.tokenizer
        total_tokens = sum(len(tokenizer.encode(t)) for t in input_texts)

        # 构建响应
        data = [
            EmbeddingData(
                embedding=emb.tolist() if hasattr(emb, 'tolist') else list(emb),
                index=i
            )
            for i, emb in enumerate(embeddings)
        ]

        return EmbeddingResponse(
            data=data,
            model=model_name,
            usage=EmbeddingUsage(
                prompt_tokens=total_tokens,
                total_tokens=total_tokens
            )
        )

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"生成 embedding 失败：{e}")
        raise HTTPException(status_code=500, detail=str(e))


@app.post("/v1/similarity")
async def calculate_similarity(request: SimilarityRequest):
    """
    计算两个文本的余弦相似度

    响应格式: {"similarity": 0.95}
    """
    if model is None:
        raise HTTPException(status_code=503, detail="模型未加载")

    try:
        emb1 = model.encode(request.text1, convert_to_tensor=True)
        emb2 = model.encode(request.text2, convert_to_tensor=True)

        sim = torch.nn.functional.cosine_similarity(
            emb1.unsqueeze(0), emb2.unsqueeze(0)
        ).item()

        return {"similarity": sim}

    except Exception as e:
        logger.error(f"计算相似度失败：{e}")
        raise HTTPException(status_code=500, detail=str(e))


# ==================== 主函数 ====================

def main():
    parser = argparse.ArgumentParser(
        description="Qwen3-Embedding 服务器 (HuggingFace + FastAPI)",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例用法:
  # 基本启动 (默认 CPU)
  python embedding_server.py

  # GPU 模式
  python embedding_server.py --device cuda

  # 指定 GPU
  python embedding_server.py --device cuda:1

  # 自定义端口和模型
  python embedding_server.py --port 8000 --model-path /path/to/model

  # 后台运行
  nohup python embedding_server.py > server.log 2>&1 &
        """
    )

    parser.add_argument(
        "--model-path",
        type=str,
        default=DEFAULT_MODEL_PATH,
        help=f"模型路径 (默认：{DEFAULT_MODEL_PATH})"
    )
    parser.add_argument(
        "--host",
        type=str,
        default=DEFAULT_HOST,
        help=f"监听地址 (默认：{DEFAULT_HOST})"
    )
    parser.add_argument(
        "--port",
        type=int,
        default=DEFAULT_PORT,
        help=f"监听端口 (默认：{DEFAULT_PORT})"
    )
    parser.add_argument(
        "--device",
        type=str,
        default="cpu",
        help="运行设备 (默认：cpu，可选：cpu, cuda, cuda:0, cuda:1 等)"
    )

    args = parser.parse_args()

    # 检查模型路径
    if not os.path.exists(args.model_path):
        logger.error(f"模型路径不存在：{args.model_path}")
        sys.exit(1)

    # 加载模型
    load_model(args.model_path, device_str=args.device)

    # 启动服务
    logger.info(f"启动服务器：http://{args.host}:{args.port}")
    uvicorn.run(
        app,
        host=args.host,
        port=args.port,
        log_level="info"
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
PI-LLM-Server 命令行入口
"""
import argparse
import sys
import os
from pathlib import Path

# 导入 FastAPI 应用和配置
from pi_llm_server.server import app, initialize_services
from pi_llm_server.config import init_config, ConfigManager
from pi_llm_server.utils.logging import init_default_logging
import uvicorn

# 默认配置目录
DEFAULT_CONFIG_DIR = Path.home() / ".config" / "pi-llm-server"
DEFAULT_CONFIG_FILE = DEFAULT_CONFIG_DIR / "config.yaml"


def ensure_config_exists() -> Path:
    """确保配置文件存在，不存在则自动创建"""
    if not DEFAULT_CONFIG_DIR.exists():
        DEFAULT_CONFIG_DIR.mkdir(parents=True, exist_ok=True)
        print(f"创建配置目录：{DEFAULT_CONFIG_DIR}")

    if not DEFAULT_CONFIG_FILE.exists():
        # 从项目目录复制示例配置
        script_dir = Path(__file__).parent
        example_config = script_dir.parent.parent / "examples" / "config.example.yaml"
        if example_config.exists():
            import shutil
            shutil.copy2(example_config, DEFAULT_CONFIG_FILE)
            print(f"创建默认配置文件：{DEFAULT_CONFIG_FILE}")
            print("请修改配置文件后重新启动")
            print(f"配置文件位置：{DEFAULT_CONFIG_FILE}")
            sys.exit(1)
        else:
            print("错误：找不到示例配置文件 examples/config.example.yaml")
            sys.exit(1)

    return DEFAULT_CONFIG_FILE


def main():
    """主函数"""
    parser = argparse.ArgumentParser(
        description="PI-LLM Server - 统一 LLM 服务网关",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例用法:
  # 使用默认配置文件（~/.config/pi-llm-server/config.yaml）
  pi-llm-server

  # 指定配置文件
  pi-llm-server --config /path/to/config.yaml

  # 指定端口和日志级别
  pi-llm-server --port 8090 --log-level debug

  # 使用 python -m 方式启动
  python -m pi_llm_server --port 8090
        """
    )

    parser.add_argument(
        "--config", "-c",
        default=None,
        help=f"配置文件路径（默认：{DEFAULT_CONFIG_FILE}）"
    )

    parser.add_argument(
        "--host",
        default=None,
        help="服务主机地址（默认：从配置文件读取）"
    )

    parser.add_argument(
        "--port", "-p",
        type=int,
        default=None,
        help="服务端口（默认：从配置文件读取）"
    )

    parser.add_argument(
        "--log-level",
        default=None,
        choices=["debug", "info", "warning", "error"],
        help="日志级别（默认：从配置文件读取）"
    )

    args = parser.parse_args()

    # 确定配置文件路径
    if args.config:
        config_path = Path(args.config)
    else:
        config_path = ensure_config_exists()

    # 检查配置文件是否存在
    if not config_path.exists():
        print(f"错误：配置文件不存在：{config_path}")
        print("请复制 config.example.yaml 并修改配置")
        sys.exit(1)

    # 初始化日志
    log_level = args.log_level or "info"
    log_dir = Path.home() / ".cache" / "pi-llm-server" / "logs"
    log_dir.mkdir(parents=True, exist_ok=True)
    init_default_logging("pi-llm-server", log_level, str(log_dir))

    import logging
    logger = logging.getLogger(__name__)

    # 加载配置
    try:
        config_manager = init_config(str(config_path))
        logger.info(f"配置文件加载成功：{config_path}")
    except Exception as e:
        logger.error(f"配置文件加载失败：{e}")
        sys.exit(1)

    # 命令行参数覆盖配置文件
    host = args.host or config_manager.config.server.host
    port = args.port or config_manager.config.server.port
    if args.log_level:
        config_manager.config.server.log_level = args.log_level

    # 初始化服务
    initialize_services(config_manager)

    # 启动服务
    logger.info(f"启动服务：http://{host}:{port}")

    # 当 workers > 1 时，uvicorn 要求传递 import string 而不是 app 对象
    # 但多 worker 模式下，每个 worker 会重新导入模块，无法共享 config_manager
    # 因此建议生产环境使用反向代理（如 nginx）实现多进程，而非 uvicorn workers
    workers = config_manager.config.server.workers
    if workers > 1:
        logger.warning(f"多 worker 模式 ({workers}) 下配置管理器无法在 worker 间共享，建议使用反向代理实现多进程。")
        logger.warning("临时使用单 worker 模式启动，如需多 worker 请使用 nginx 等反向代理")
        workers = 1  # 临时降级为单 worker

    # 单 worker 模式，直接传递 app 对象
    uvicorn.run(
        app,
        host=host,
        port=port,
        log_level=config_manager.config.server.log_level,
        workers=workers,
    )


if __name__ == "__main__":
    main()



"""
允许通过 python -m pi_llm_server 启动服务

用法:
    # 仅启动统一网关（默认）
    python -m pi_llm_server

    # 启动所有后台服务
    python -m pi_llm_server services start --all

    # 启动后台服务 + 网关（一站式启动）
    python -m pi_llm_server start-all

    # 启动单个后台服务
    python -m pi_llm_server services start embedding

    # 查看后台服务状态
    python -m pi_llm_server services status

    # 查看网关 + 后台服务状态
    python -m pi_llm_server status

    # 使用 CLI 参数启动网关
    python -m pi_llm_server --port 8090
"""
import sys
import subprocess
import time
from pathlib import Path


# 默认配置目录
DEFAULT_CONFIG_DIR = Path.home() / ".config" / "pi-llm-server"
DEFAULT_CONFIG_FILE = DEFAULT_CONFIG_DIR / "config.yaml"


def ensure_config_exists() -> Path:
    """确保配置文件存在，不存在则自动创建"""
    if not DEFAULT_CONFIG_DIR.exists():
        DEFAULT_CONFIG_DIR.mkdir(parents=True, exist_ok=True)
        print(f"创建配置目录：{DEFAULT_CONFIG_DIR}")

    if not DEFAULT_CONFIG_FILE.exists():
        # 从项目目录复制示例配置
        example_config = Path(__file__).parent.parent / "examples" / "config.example.yaml"
        if example_config.exists():
            import shutil
            shutil.copy2(str(example_config), str(DEFAULT_CONFIG_FILE))
            print(f"创建默认配置文件：{DEFAULT_CONFIG_FILE}")
            print("请修改配置文件后重新启动")
            print(f"配置文件位置：{DEFAULT_CONFIG_FILE}")
            sys.exit(1)
        else:
            print("错误：找不到示例配置文件 examples/config.example.yaml")
            sys.exit(1)

    return DEFAULT_CONFIG_FILE


def start_gateway(background: bool = False, config_file: Path = None):
    """启动网关服务"""
    config_file = config_file or ensure_config_exists()

    if background:
        log_dir = Path.home() / ".cache" / "pi-llm-server" / "logs"
        log_dir.mkdir(parents=True, exist_ok=True)
        log_file = log_dir / "gateway.log"

        pid_dir = Path.home() / ".cache" / "pi-llm-server" / "pids"
        pid_dir.mkdir(parents=True, exist_ok=True)
        pid_file = pid_dir / "gateway.pid"

        with open(log_file, 'a') as f:
            proc = subprocess.Popen(
                [sys.executable, '-m', 'pi_llm_server', '--config', str(config_file)],
                stdout=f,
                stderr=f,
                start_new_session=True,
            )
        pid_file.write_text(str(proc.pid))
        print(f"✓ 网关已启动 (PID: {proc.pid})")
        time.sleep(2)
        return True
    else:
        # 前台启动，直接调用 CLI
        from pi_llm_server.cli import main as cli_main
        cli_main()
        return True


def stop_gateway() -> bool:
    """停止网关服务"""
    import signal
    import os

    pid_dir = Path.home() / ".cache" / "pi-llm-server" / "pids"
    pid_file = pid_dir / "gateway.pid"

    if pid_file.exists():
        try:
            pid = int(pid_file.read_text().strip())
            os.kill(pid, signal.SIGTERM)
            print(f"发送 SIGTERM 到网关 (PID: {pid})")

            # 等待进程终止
            for _ in range(10):
                time.sleep(0.5)
                try:
                    os.kill(pid, 0)
                except OSError:
                    break
            else:
                os.kill(pid, signal.SIGKILL)
                print(f"发送 SIGKILL 到网关")

            pid_file.unlink()
            print("✓ 网关已停止")
            return True

        except ProcessLookupError:
            pid_file.unlink(missing_ok=True)
            print("✓ 网关已停止")
            return True
        except Exception as e:
            print(f"✗ 停止网关失败：{e}")
            return False

    # 尝试通过端口检查
    import socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        result = sock.connect_ex(('127.0.0.1', 8090))
        sock.close()
        if result == 0:
            print("⚠ 网关在运行但没有 PID 文件，请手动停止")
            return False
    except Exception:
        pass

    print("✓ 网关未运行")
    return True


def is_gateway_running() -> bool:
    """检查网关是否正在运行"""
    import socket

    # 方法 1: 检查 PID 文件
    pid_dir = Path.home() / ".cache" / "pi-llm-server" / "pids"
    pid_file = pid_dir / "gateway.pid"

    if pid_file.exists():
        try:
            pid = int(pid_file.read_text().strip())
            import os
            os.kill(pid, 0)
            return True
        except (ValueError, OSError):
            pid_file.unlink(missing_ok=True)

    # 方法 2: 检查端口
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        result = sock.connect_ex(('127.0.0.1', 8090))
        sock.close()
        return result == 0
    except Exception:
        return False


def show_full_status():
    """显示网关 + 后台服务状态"""
    print("=" * 60)
    print("PI-LLM-Server 全部服务状态")
    print("=" * 60)
    print()

    # 网关状态
    gateway_status = "运行中" if is_gateway_running() else "已停止"
    gateway_symbol = "✓" if is_gateway_running() else "✗"
    print(f"  {gateway_symbol} {'统一网关':20s} {gateway_status:10s} 端口：8090")

    # 后台服务状态
    from pi_llm_server.launcher.service_manager import SERVICE_CONFIG, is_service_running, get_service_pid

    for name, cfg in SERVICE_CONFIG.items():
        status = "运行中" if is_service_running(name) else "已停止"
        pid = get_service_pid(name)
        pid_str = f"(PID: {pid})" if pid else ""
        symbol = "✓" if is_service_running(name) else "✗"
        print(f"  {symbol} {cfg['name']:20s} {status:10s} 端口：{cfg['port']:5d} {pid_str}")

    print()
    print("=" * 60)


def main():
    """主入口函数"""
    # 检查子命令
    if len(sys.argv) > 1:
        if sys.argv[1] == 'services':
            # 传递给 service_manager（只管理后台服务）
            sys.argv.pop(1)
            from pi_llm_server.launcher.service_manager import main as service_main
            service_main()
        elif sys.argv[1] == 'start-all':
            # 一站式启动：后台服务 + 网关
            print("=" * 60)
            print("启动所有服务（后台服务 + 网关）")
            print("=" * 60)
            print()

            # 确保配置文件存在
            config_file = ensure_config_exists()

            # 启动后台服务
            from pi_llm_server.launcher.service_manager import start_all
            start_all()

            print()
            print("-" * 60)
            print()

            # 启动网关
            start_gateway(background=True, config_file=config_file)

            print()
            print("=" * 60)
            print("所有服务已启动")
            print("=" * 60)
        elif sys.argv[1] == 'stop-all':
            # 一站式停止：网关 + 后台服务
            print("=" * 60)
            print("停止所有服务")
            print("=" * 60)
            print()

            # 先停止网关
            stop_gateway()

            print()

            # 停止后台服务
            from pi_llm_server.launcher.service_manager import stop_all
            stop_all()

            print()
            print("=" * 60)
            print("所有服务已停止")
            print("=" * 60)
        elif sys.argv[1] == 'status':
            # 显示全部服务状态
            show_full_status()
        else:
            # 默认启动网关
            from pi_llm_server.cli import main as cli_main
            cli_main()
    else:
        # 默认启动网关
        from pi_llm_server.cli import main as cli_main
        cli_main()


if __name__ == "__main__":
    main()
